/*
 * This file is part of the LibreOffice project.
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * This file incorporates work covered by the following license notice:
 *
 *   Licensed to the Apache Software Foundation (ASF) under one or more
 *   contributor license agreements. See the NOTICE file distributed
 *   with this work for additional information regarding copyright
 *   ownership. The ASF licenses this file to you under the Apache
 *   License, Version 2.0 (the "License"); you may not use this file
 *   except in compliance with the License. You may obtain a copy of
 *   the License at http://www.apache.org/licenses/LICENSE-2.0 .
 */

/*
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
+                          OPENOFFICE.ORG 2.0 HELP                  +
+                       HIGH CONTRAST #1 STYLESHEET                 +
+                           WESTERN LANGUAGES                       +
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
+ LAST CHANGES: 15-NOV-2004                                         +
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*/


body, p, h1, h2, h3, h4, h5, h6, .listitem, .listitemintable, .tablecontent, .tablecontentintable
	{ font-family: "Lohit Hindi","Albany AMT",Albany,Arial,"Nimbus Sans L","Bitstream Vera Sans",Helvetica,Lucida,Geneva,Helmet,"Andale Sans UI","Arial Unicode MS","Lucida Sans Unicode",Tahoma,sans-serif; }

pre, .code, .codeintable, .example, .exampleintable, .literal, .literalintable, .path, .pathintable
	{ font-family: "Lohit Hindi","Cumberland AMT",Cumberland,"Courier New","Nimbus Mono L","Bitstream Vera Sans Mono",Courier,"Lucida Sans Typewriter","Lucida Typewriter",Monaco,Monospaced; margin-top: 1pt; margin-bottom: 1pt;}

.acronym 
	{ font-weight: bold;}

.related
	{ font-weight: bold; margin-top:20pt;}

.emph, .menuitem, .keycode
	{ font-weight: bold; }

.tablehead, .tableheadintable
	{ font-weight: bold; margin-top: 0px;}

.howtogetheader
	{ font-weight: bold; padding: 3px;}

h1, h2, h3, h4, h5, h6
	{ margin-bottom: 5pt; }

p, td
	{ font-size: 10pt; }

h1
	{ font-size: 18pt; padding-bottom: 6px; margin-bottom: 6px;}

h2
	{ font-size: 14pt; }

h3
	{ font-size: 12pt; }

h4, h5, h6
	{ font-size: 10pt; }

.relatedtopics
	{ font-weight: normal; }

.relatedbody  
	{ margin-top: 2px; margin-bottom: 2px; margin-left: 5px; }

.wide 
	{ width: 100%; }

.topalign
	 { vertical-align: top; border: 1px;}

.bug 
	{ color: red; border: 1px solid red;}


/* HIGH CONTRAST SPECIFIC SETTINGS */

body, p, h1, h2, h3, h4, h5, h6, .listitem, .listitemintable, .tablecontent, .tablecontentintable
	{ background: #000000; color: #FFFF00;}

.related 
	{ border-top: 1px solid #FFFF00; }

.howtogetheader
	{border: 1px solid #EFEFEF; background: #000000;}

h1 
	{ border-bottom: 1px solid #FFFF00; ]

.howtoget
	{ background:#000000;}

.howtogetbody
	{ background:#000000;}

.debug
	{ border: 1px solid red; padding: 3px; display: none;}

a
 { color: #FFFF00; }